@@ -48,8 +48,8 @@ module Bustle.Diagram
 where
 
 import Data.List (unzip4)
+import Data.List.NonEmpty (NonEmpty(..), toList)
 import Control.Arrow ((&&&))
-import Control.Applicative ((<$>), (<*>))
 
 import Control.Monad.Reader
 
@@ -60,7 +60,6 @@ import Graphics.Rendering.Pango.Font
 
 import qualified Bustle.Marquee as Marquee
 import Bustle.Marquee (Marquee)
-import Bustle.Util
 import Bustle.Types (ObjectPath, InterfaceName, MemberName)
 
 -- Sorry Mum
@@ -131,15 +130,15 @@ memberLabel :: ObjectPath
             -> Bool   -- ^ True if this is a return; False if it's a call
             -> Double -- ^ y-coordinate
             -> Shape
-memberLabel p i m isReturn y = MemberLabel p i m isReturn memberx y
+memberLabel p i m isReturn = MemberLabel p i m isReturn memberx
 
 timestampLabel :: String -> Double -> Shape
-timestampLabel s y = TimestampLabel s timestampx y
+timestampLabel s = TimestampLabel s timestampx
 
 type Diagram = [Shape]
 
 arcControlPoints :: Shape -> (Point, Point)
-arcControlPoints (Arc { topx=x1, topy=y1, bottomx=x2, bottomy=y2, arcside=s }) =
+arcControlPoints Arc { topx=x1, topy=y1, bottomx=x2, bottomy=y2, arcside=s } =
     let (+-) = offset s
         cp1 = (x1 +- 60, y1 + 10)
         cp2 = (x2 +- 60, y2 - 10)
@@ -161,7 +160,7 @@ mapX f s = case s of
     Arc {}         -> s { topx = f (topx s)
                         , bottomx = f (bottomx s)
                         }
-    ClientLines {} -> s { shapexs = mapNonEmpty f (shapexs s) }
+    ClientLines {} -> s { shapexs = f <$> shapexs s }
     _              -> s { shapex = f (shapex s) }
 
 mapY f s = case s of
@@ -220,7 +219,7 @@ headerHeight = fromIntegral . (10 *) . length
 bounds :: Shape -> Rect
 bounds s = case s of
   ClientLines {} ->
-    let xs = nonEmptyToList (shapexs s)
+    let xs = toList (shapexs s)
     in  (minimum xs, shapey1 s, maximum xs, shapey2 s)
   Rule {} -> (shapex1 s, shapey s, shapex2 s, shapey s)
   Arrow {} ->
@@ -230,11 +229,11 @@ bounds s = case s of
     in (x1, y1, x2, y2)
   SignalArrow {} ->
     let (x1, x2) = xMinMax s
-        (y1, y2) = (subtract 5) &&& (+5) $ shapey s
+        (y1, y2) = subtract 5 &&& (+5) $ shapey s
     in (x1, y1, x2, y2)
   DirectedSignalArrow {} ->
     let (x1, x2) = minMax (epicentre s, shapex s)
-        (y1, y2) = (subtract 5) &&& (+5) $ shapey s
+        (y1, y2) = subtract 5 &&& (+5) $ shapey s
     in (x1, y1, x2, y2)
   Arc { topx=x1, bottomx=x2, topy=y1, bottomy=y2 } ->
     let ((cx, _), (dx, _)) = arcControlPoints s
@@ -270,11 +269,12 @@ headers xss y = (height, shapes)
 --
 
 diagramBounds :: Diagram -> ((Double, Double), (Double, Double))
-diagramBounds shapes = ((minimum (0:x1s), minimum (0:y1s))
-                       ,(maximum (0:x2s), maximum (0:y2s))
+diagramBounds shapes = ((minimum (0:x1s) - padding, minimum (0:y1s) - padding)
+                       ,(maximum (0:x2s) + padding, maximum (0:y2s) + padding)
                        )
   where
     (x1s, y1s, x2s, y2s) = unzip4 $ map bounds shapes
+    padding              = 6
 
 diagramDimensions :: Diagram -> (Double, Double)
 diagramDimensions shapes = (x2 - x1, y2 - y1)
@@ -405,13 +405,13 @@ drawSignalArrow e mleft mright y = do
     arc e y 5 0 (2 * pi)
     stroke
 
-    maybeM mleft $ \left -> do
+    forM_ mleft $ \left -> do
         moveTo left y
         arrowHead False
         lineTo (e - 5) y
         stroke
 
-    maybeM mright $ \right -> do
+    forM_ mright $ \right -> do
         moveTo (e + 5) y
         lineTo right y
         arrowHead True
@@ -503,7 +503,7 @@ drawTimestamp ts x y = do
 drawClientLines :: NonEmpty Double -> Double -> Double -> Render ()
 drawClientLines xs y1 y2 = saved $ do
     setSourceRGB 0.7 0.7 0.7
-    forM_ (nonEmptyToList xs) $ \x -> do
+    forM_ (toList xs) $ \x -> do
         moveTo x y1
         lineTo x y2
         stroke
