@@ -44,7 +44,6 @@ module Bustle.Types
   , fakeUniqueName
 
   , Microseconds(..)
-  , µsFromPair
   , µsToMs
 
   , Member(..)
@@ -113,11 +112,6 @@ fakeUniqueName = UniqueName . busName_ . (":fake." ++)
 newtype Microseconds = Microseconds Integer
   deriving (Show, Ord, Eq, Num, Real, Enum, Integral)
 
-µsFromPair :: Integer
-           -> Integer
-           -> Microseconds
-µsFromPair s µs = Microseconds $ (s * 1000000) + µs
-
 µsToMs :: Microseconds
        -> Integer
 µsToMs (Microseconds µs) = µs `div` 1000
@@ -165,14 +159,15 @@ type MessageSize = Int
 data Detailed e =
     Detailed { deTimestamp :: Microseconds
              , deEvent :: e
-             , deDetails :: Maybe (MessageSize, ReceivedMessage)
+             , deMessageSize :: MessageSize
+             , deReceivedMessage :: ReceivedMessage
              }
   deriving (Show, Eq, Functor)
 
 type DetailedEvent = Detailed Event
 
 instance Ord e => Ord (Detailed e) where
-    compare (Detailed µs x _) (Detailed µs' y _)
+    compare (Detailed µs x _ _) (Detailed µs' y _ _)
         = compare (µs, x) (µs', y)
 
 data Change = Claimed UniqueName
@@ -184,10 +179,10 @@ partitionDetaileds :: [DetailedEvent]
                    -> ([Detailed NOC], [Detailed Message])
 partitionDetaileds = partitionEithers . map f
   where
-    f (Detailed µs e details) =
+    f (Detailed µs e size rm) =
         case e of
-            NOCEvent n -> Left $ Detailed µs n details
-            MessageEvent m -> Right $ Detailed µs m details
+            NOCEvent n -> Left $ Detailed µs n size rm
+            MessageEvent m -> Right $ Detailed µs m size rm
 
 mentionedNames :: Message -> [TaggedBusName]
 mentionedNames m = sender m:dest
