@@ -1,6 +1,7 @@
 {-
 Bustle.UI: displays charts of D-Bus activity
 Copyright © 2008–2011 Collabora Ltd.
+Copyright © 2018      Will Thompson
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,14 +23,17 @@ module Bustle.UI
   )
 where
 
+import Control.Monad (void)
 import Control.Monad.Reader
 import Control.Monad.State
 import Control.Monad.Except
 
 import Data.IORef
+import qualified Data.Map as Map
 import qualified Data.Set as Set
 import Data.List (intercalate)
 import Data.Time
+import Data.Tuple (swap)
 import Data.Monoid (mempty)
 import Text.Printf
 
@@ -38,7 +42,8 @@ import Bustle.Application.Monad
 import Bustle.Renderer
 import Bustle.Types
 import Bustle.Diagram
-import Bustle.Marquee (toString)
+import qualified Bustle.Marquee as Marquee
+import Bustle.Monitor
 import Bustle.Util
 import Bustle.UI.AboutDialog
 import Bustle.UI.Canvas
@@ -46,14 +51,15 @@ import Bustle.UI.DetailsView
 import Bustle.UI.FilterDialog
 import Bustle.UI.OpenTwoDialog (setupOpenTwoDialog)
 import Bustle.UI.Recorder
-import Bustle.UI.Util (displayError)
+import Bustle.UI.RecordAddressDialog (showRecordAddressDialog)
 import Bustle.StatisticsPane
 import Bustle.Translation (__)
 import Bustle.Loader
+import Bustle.Loader.Pcap (convert)
 
 import qualified Control.Exception as C
 import System.Glib.GError (GError(..), failOnGError)
-import System.Glib.Properties (objectSetPropertyMaybeString)
+import System.GIO.Enums (IOErrorEnum(IoErrorCancelled))
 
 import Graphics.UI.Gtk
 
@@ -72,34 +78,42 @@ data LogDetails =
   | SingleLog FilePath
   | TwoLogs FilePath FilePath
 
+-- Must be kept in sync with the names in the GtkBuilder file
 data Page =
     InstructionsPage
   | PleaseHoldPage
   | CanvasPage
   deriving
-    (Enum)
+    (Show)
 
 data WindowInfo =
     WindowInfo { wiWindow :: Window
-               , wiHeaderBar :: Widget -- TODO
+               , wiTitle :: Label
+               , wiSubtitle :: Label
+               , wiSpinner :: Spinner
+               , wiRecord :: Button
+               , wiStop :: Button
+               , wiOpen :: Button
                , wiSave :: Button
                , wiExport :: Button
                , wiViewStatistics :: CheckMenuItem
                , wiFilterNames :: MenuItem
-               , wiNotebook :: Notebook
-               , wiStatsBook :: Notebook
+               , wiErrorBar :: InfoBar
+               , wiErrorBarTitle :: Label
+               , wiErrorBarDetails :: Label
+               , wiStack :: Stack
+               , wiSidebarHeader :: Widget -- TODO, GtkHeaderBar
+               , wiSidebarStack :: Stack
                , wiStatsPane :: StatsPane
-               , wiContentVPaned :: VPaned
+               , wiContentPaned :: Paned
                , wiCanvas :: Canvas (Detailed Message)
                , wiDetailsView :: DetailsView
 
                , wiLogDetails :: IORef (Maybe LogDetails)
                }
 
-data BConfig =
+newtype BConfig =
     BConfig { debugEnabled :: Bool
-            , methodIcon :: Maybe Pixbuf
-            , signalIcon :: Maybe Pixbuf
             }
 
 data BState = BState { windows :: Int
@@ -122,12 +136,7 @@ uiMain = failOnGError $ do
     -- FIXME: get a real option parser
     let debug = any isDebug args
 
-    [method, signal] <- mapM loadPixbuf
-        ["dfeet-method.png", "dfeet-signal.png"]
-
     let config = BConfig { debugEnabled = debug
-                         , methodIcon = method
-                         , signalIcon = signal
                          }
         initialState = BState { windows = 0
                               , initialWindow = Nothing
@@ -153,8 +162,8 @@ mainB args = do
 
 createInitialWindow :: B ()
 createInitialWindow = do
-  misc <- emptyWindow
-  modify $ \s -> s { initialWindow = Just misc }
+  wi <- emptyWindow
+  putInitialWindow wi
 
 consumeInitialWindow :: B WindowInfo
 consumeInitialWindow = do
@@ -165,6 +174,11 @@ consumeInitialWindow = do
             modify $ \s -> s { initialWindow = Nothing }
             return windowInfo
 
+putInitialWindow :: WindowInfo
+                 -> B ()
+putInitialWindow wi =
+    modify $ \s -> s { initialWindow = Just wi }
+
 loadInInitialWindow :: LogDetails -> B ()
 loadInInitialWindow = loadLogWith consumeInitialWindow
 
@@ -191,6 +205,8 @@ loadLogWith :: B WindowInfo   -- ^ action returning a window to load the log(s)
             -> LogDetails
             -> B ()
 loadLogWith getWindow logDetails = do
+    windowInfo <- getWindow
+
     ret <- runExceptT $ do
         ((sessionWarnings, sessionMessages),
          (systemWarnings, systemMessages)) <- openLog logDetails
@@ -199,7 +215,6 @@ loadLogWith getWindow logDetails = do
         let rr = process sessionMessages systemMessages
         io $ mapM warn $ sessionWarnings ++ systemWarnings ++ rrWarnings rr
 
-        windowInfo <- lift getWindow
         lift $ displayLog windowInfo
                           logDetails
                           sessionMessages
@@ -207,27 +222,147 @@ loadLogWith getWindow logDetails = do
                           rr
 
     case ret of
-      Left (LoadError f e) -> io $
-          displayError Nothing (printf (__ "Could not read '%s'") f) (Just e)
-      Right () -> return ()
+      Left (LoadError f e) -> do
+          let title = printf (__ "Could not read '%s'") f
+          io $ displayError windowInfo title (Just e)
+          putInitialWindow windowInfo
+      Right () ->
+          io $ hideError windowInfo
 
-startRecording :: B ()
-startRecording = do
+    io $ windowPresent (wiWindow windowInfo)
+
+
+updateRecordingSubtitle :: WindowInfo
+                        -> Int
+                        -> IO ()
+updateRecordingSubtitle wi j = do
+    let message = printf (__ "Logged <b>%u</b> messages") j :: String
+    labelSetMarkup (wiSubtitle wi) message
+
+
+processBatch :: IORef [DetailedEvent]
+             -> IORef Int
+             -> WindowInfo
+             -> IO (IO Bool)
+processBatch pendingRef n wi = do
+    rendererStateRef <- newIORef rendererStateNew
+    -- FIXME: this is stupid. If we have to manually combine the outputs, it's
+    -- basically just more state.
+    rendererResultRef <- newIORef mempty
+
+    return $ do
+        pending <- readIORef pendingRef
+        writeIORef pendingRef []
+
+        unless (null pending) $ do
+            rr <- atomicModifyIORef' rendererStateRef $ \s ->
+                swap $ processSome (reverse pending) [] s
+
+            oldRR <- readIORef rendererResultRef
+            let rr' = oldRR `mappend` rr
+            writeIORef rendererResultRef rr'
+
+            unless (null (rrShapes rr)) $ do
+                -- If the renderer produced some visible output, count it as a
+                -- message from the user's perspective.
+                modifyIORef' n (+ length pending)
+                j <- readIORef n
+                updateRecordingSubtitle wi j
+
+                aChallengerAppears wi rr'
+
+        return True
+
+
+recorderRun :: WindowInfo
+            -> Either BusType String
+            -> FilePath
+            -> BustleEnv BConfig BState
+            -> IO ()
+recorderRun wi target filename r = C.handle newFailed $ do
+    -- TODO: I'm pretty sure the monitor is leaked
+    monitor <- monitorNew target filename
+    loaderStateRef <- newIORef Map.empty
+    pendingRef <- newIORef []
+
+    let updateLabel µs body = do
+            s <- readIORef loaderStateRef
+            (m, s') <- runStateT (convert µs body) s
+            s' `seq` writeIORef loaderStateRef s'
+
+            case m of
+                Left e -> warn e
+                Right message
+                  | isRelevant (deEvent message) ->
+                        modifyIORef' pendingRef (message:)
+                  | otherwise -> return ()
+
+    n <- newIORef (0 :: Int)
+    processor <- processBatch pendingRef n wi
+    processorId <- timeoutAdd processor 200
+
+    stopActivatedId <- wiStop wi `on` buttonActivated $ monitorStop monitor
+    handlerId <- monitor `on` monitorMessageLogged $ updateLabel
+    void $ monitor `on` monitorStopped $ \domain code message -> do
+        handleError domain code message
+
+        signalDisconnect stopActivatedId
+        signalDisconnect handlerId
+
+        -- Flush out any last messages from the queue.
+        timeoutRemove processorId
+        processor
+
+        hadOutput <- fmap (/= 0) (readIORef n)
+        finished hadOutput
+  where
+    newFailed (GError domain code message) = do
+        finished False
+        handleError domain code message
+
+    finished hadOutput =
+        makeCallback (finishedRecording wi filename hadOutput) r
+
+    -- Filter out IoErrorCancelled. In theory one should use
+    --   catchGErrorJust IoErrorCancelled computation (\_ -> return ())
+    -- but IOErrorEnum does not have an instance for GError domain.
+    handleError domain code message = do
+        gIoErrorQuark <- quarkFromString "g-io-error-quark"
+        let cancelled = fromEnum IoErrorCancelled
+        unless (domain == gIoErrorQuark && code == cancelled) $
+            displayError wi (Marquee.toString message) Nothing
+
+
+startRecording :: Either BusType String
+               -> B ()
+startRecording target = do
     wi <- consumeInitialWindow
 
-    zt <- io $ getZonedTime
+    zt <- io getZonedTime
     -- I hate time manipulation
     let yyyy_mm_dd_hh_mm_ss = takeWhile (/= '.') (show zt)
 
-    cacheDir <- io $ getCacheDir
-    let filename = cacheDir </> yyyy_mm_dd_hh_mm_ss <.> "bustle"
+    cacheDir <- io getCacheDir
+    let filename = cacheDir </> yyyy_mm_dd_hh_mm_ss <.> "pcap"
+
+    let title = printf (__ "Recording %s&#8230;") $ case target of
+            Left BusTypeNone    -> error "whoops, this value shouldn't exist"
+            Left BusTypeSession -> "session bus"
+            Left BusTypeSystem  -> "system bus"
+            Right address       -> address
+
+    io $ do
+        hideError wi
+        widgetHide (wiRecord wi)
+        widgetHide (wiOpen wi)
+        widgetShow (wiStop wi)
+        widgetGrabFocus (wiStop wi)
+        spinnerStart (wiSpinner wi)
+        labelSetMarkup (wiTitle wi) (title :: String)
+        updateRecordingSubtitle wi 0
 
     setPage wi PleaseHoldPage
-    let mwindow = Just (wiWindow wi)
-        progress = aChallengerAppears wi
-        finished = finishedRecording wi filename
-    embedIO $ \r -> recorderRun filename mwindow progress
-                                (\p -> makeCallback (finished p) r)
+    embedIO $ recorderRun wi target filename
 
 aChallengerAppears :: WindowInfo
                    -> RendererResult a
@@ -241,16 +376,21 @@ onMenuItemActivate :: MenuItemClass menuItem
                    => menuItem
                    -> IO ()
                    -> IO (ConnectId menuItem)
-onMenuItemActivate mi act =
-    on mi menuItemActivate act
+onMenuItemActivate mi = on mi menuItemActivate
 
 finishedRecording :: WindowInfo
                   -> FilePath
                   -> Bool
                   -> B ()
 finishedRecording wi tempFilePath producedOutput = do
+    io $ do
+        widgetShow (wiRecord wi)
+        widgetShow (wiOpen wi)
+        widgetHide (wiStop wi)
+        spinnerStop (wiSpinner wi)
+
     if producedOutput
-      then do
+      then void $ do
         -- TODO: There is a noticable lag when reloading big files. It would be
         -- nice to either make the loading faster, or eliminate the reload.
         loadLogWith (return wi) (RecordedLog tempFilePath)
@@ -260,11 +400,13 @@ finishedRecording wi tempFilePath producedOutput = do
         io $ do
             widgetSetSensitivity saveItem True
             saveItem `on` buttonActivated $ showSaveDialog wi (return ())
-        return ()
       else do
         setPage wi InstructionsPage
-        modify $ \s -> s { initialWindow = Just wi }
+        putInitialWindow wi
         updateDisplayedLog wi (mempty :: RendererResult ())
+        io $ do
+            wiTitle wi `set` [ labelText := "" ]
+            wiSubtitle wi `set` [ labelText := "" ]
 
 showSaveDialog :: WindowInfo
                -> IO ()
@@ -278,22 +420,19 @@ showSaveDialog wi savedCb = do
         let tempFile = fileFromParseName tempFilePath
         let newFile  = fileFromParseName newFilePath
 
-        C.catch (fileMove tempFile newFile [FileCopyOverwrite] Nothing Nothing) $ \(GError _ _ msg) -> do
-            d <- messageDialogNew mwindow [DialogModal] MessageError ButtonsOk (__ "Couldn't save log")
-            let secondary :: String
-                secondary = printf
-                    (__ "Error: <i>%s</i>\n\n\
-                        \You might want to manually recover the log from the temporary file at\n\
-                        \<tt>%s</tt>") (toString msg) tempFilePath
-            messageDialogSetSecondaryMarkup d secondary
-            widgetShowAll d
-            d `after` response $ \_ -> do
-                widgetDestroy d
-            return ()
-
-        widgetSetSensitivity (wiSave wi) False
-        wiSetLogDetails wi (SingleLog newFilePath)
-        savedCb
+        result <- C.try $ fileMove tempFile newFile [FileCopyOverwrite] Nothing Nothing
+        case result of
+            Right _ -> do
+                widgetSetSensitivity (wiSave wi) False
+                wiSetLogDetails wi (SingleLog newFilePath)
+                hideError wi
+                savedCb
+            Left (GError _ _ msg) -> do
+                let title = __ "Couldn't save log: " ++ Marquee.toString msg
+                    secondary = printf
+                        (__ "You might want to manually recover the log from the temporary file at \
+                            \\"%s\".") tempFilePath
+                displayError wi title (Just secondary)
 
 -- | Show a confirmation dialog if the log is unsaved. Suitable for use as a
 --   'delete-event' handler.
@@ -344,20 +483,36 @@ emptyWindow = do
   let getW cast name = io $ builderGetObject builder cast name
 
   window <- getW castToWindow "diagramWindow"
-  header <- getW castToWidget "header"
+
+  title    <- getW castToLabel "headerTitle"
+  subtitle <- getW castToLabel "headerSubtitle"
+  spinner  <- getW castToSpinner "headerSpinner"
 
   [openItem, openTwoItem] <- mapM (getW castToMenuItem) ["open", "openTwo"]
-  [headerNew, headerSave, headerExport] <- mapM (getW castToButton) ["headerNew", "headerSave", "headerExport"]
+  recordSessionItem <- getW castToMenuItem "recordSession"
+  recordSystemItem <- getW castToMenuItem "recordSystem"
+  recordAddressItem <- getW castToMenuItem "recordAddress"
+  headerRecord <- getW castToButton "headerRecord"
+  headerStop   <- getW castToButton "headerStop"
+  headerOpen   <- getW castToButton "headerOpen"
+  headerSave   <- getW castToButton "headerSave"
+  headerExport <- getW castToButton "headerExport"
 
   viewStatistics <- getW castToCheckMenuItem "statistics"
   filterNames <- getW castToMenuItem "filter"
   aboutItem <- getW castToMenuItem "about"
 
-  [newButton, openButton] <- mapM (getW castToButton) ["newButton", "openButton"]
+  errorBar        <- getW castToInfoBar "errorBar"
+  errorBarTitle   <- getW castToLabel "errorBarTitle"
+  errorBarDetails <- getW castToLabel "errorBarDetails"
 
-  [nb, statsBook] <- mapM (getW castToNotebook)
-      ["diagramOrNot", "statsBook"]
-  contentVPaned <- getW castToVPaned "contentVPaned"
+  io $ errorBar `on` infoBarResponse $ \_ ->
+      widgetHide errorBar
+
+  stack <- getW castToStack "diagramOrNot"
+  sidebarHeader <- getW castToWidget "sidebarHeader"
+  sidebarStack <- getW castToStack "sidebarStack"
+  contentPaned <- getW castToPaned "contentPaned"
 
   -- Open two logs dialog
   openTwoDialog <- embedIO $ \r ->
@@ -369,14 +524,15 @@ emptyWindow = do
 
   -- File menu and related buttons
   embedIO $ \r -> do
-      let new = makeCallback startRecording r
-      forM [headerNew, newButton] $ \button ->
-          button `on` buttonActivated $ new
-
-      let open = makeCallback openDialogue r
-      onMenuItemActivate openItem open
-      openButton `on` buttonActivated $ open
+      onMenuItemActivate recordSessionItem $
+          makeCallback (startRecording (Left BusTypeSession)) r
+      onMenuItemActivate recordSystemItem $
+          makeCallback (startRecording (Left BusTypeSystem)) r
+      onMenuItemActivate recordAddressItem $
+          showRecordAddressDialog window $ \address ->
+              makeCallback (startRecording (Right address)) r
 
+      onMenuItemActivate openItem $ makeCallback (showOpenDialog window) r
       onMenuItemActivate openTwoItem $ widgetShowAll openTwoDialog
 
   -- TODO: really this wants to live in the application menu, but that entails binding GApplication,
@@ -385,35 +541,36 @@ emptyWindow = do
   -- Similarly, the drop-down menus would look better as popovers. But here we are.
   io $ onMenuItemActivate aboutItem $ showAboutDialog window
 
-  m <- asks methodIcon
-  s <- asks signalIcon
-  statsPane <- io $ statsPaneNew builder m s
+  statsPane <- io $ statsPaneNew builder
 
-  details <- io $ detailsViewNew
-  io $ do
-      let top = detailsViewGetTop details
-      panedPack2 contentVPaned top False False
-      -- Hide the details by default; they'll be shown when the user selects a
-      -- message.
-      widgetHide top
+  details <- io $ detailsViewNew builder
 
   -- The stats start off hidden.
-  io $ widgetHide statsBook
+  io $ widgetHide sidebarStack
 
   showBounds <- asks debugEnabled
   canvas <- io $ canvasNew builder showBounds (updateDetailsView details)
 
   logDetailsRef <- io $ newIORef Nothing
   let windowInfo = WindowInfo { wiWindow = window
-                              , wiHeaderBar = header
+                              , wiTitle = title
+                              , wiSubtitle = subtitle
+                              , wiSpinner = spinner
+                              , wiRecord = headerRecord
+                              , wiOpen   = headerOpen
+                              , wiStop = headerStop
                               , wiSave = headerSave
                               , wiExport = headerExport
                               , wiViewStatistics = viewStatistics
                               , wiFilterNames = filterNames
-                              , wiNotebook = nb
-                              , wiStatsBook = statsBook
+                              , wiErrorBar = errorBar
+                              , wiErrorBarTitle = errorBarTitle
+                              , wiErrorBarDetails = errorBarDetails
+                              , wiStack = stack
+                              , wiSidebarHeader = sidebarHeader
+                              , wiSidebarStack = sidebarStack
                               , wiStatsPane = statsPane
-                              , wiContentVPaned = contentVPaned
+                              , wiContentPaned = contentPaned
                               , wiCanvas = canvas
                               , wiDetailsView = details
                               , wiLogDetails = logDetailsRef
@@ -427,9 +584,9 @@ emptyWindow = do
 updateDetailsView :: DetailsView
                   -> Maybe (Detailed Message)
                   -> IO ()
-updateDetailsView detailsView newMessage = do
+updateDetailsView detailsView newMessage =
     case newMessage of
-        Nothing -> do
+        Nothing ->
             widgetHide $ detailsViewGetTop detailsView
         Just m  -> do
             detailsViewUpdate detailsView m
@@ -455,15 +612,15 @@ splitFileName_ s = (dropTrailingPathSeparator d, f)
       (d, f) = splitFileName s
 
 logWindowTitle :: LogDetails
-               -> (String, Maybe String)
-logWindowTitle (RecordedLog filepath) = ("*" ++ takeFileName filepath, Nothing)
-logWindowTitle (SingleLog   filepath) = (name, Just directory)
+               -> (String, String)
+logWindowTitle (RecordedLog filepath) = ("*" ++ takeFileName filepath, "")
+logWindowTitle (SingleLog   filepath) = (name, directory)
   where
     (directory, name) = splitFileName_ filepath
 logWindowTitle (TwoLogs sessionPath systemPath) =
     -- TODO: this looks terrible, need a custom widget
     (sessionName ++ " & " ++ systemName,
-     Just $ if sessionDirectory == systemDirectory
+     if sessionDirectory == systemDirectory
         then sessionDirectory
         else sessionDirectory ++ " & " ++ systemDirectory)
   where
@@ -484,15 +641,15 @@ wiSetLogDetails :: WindowInfo
 wiSetLogDetails wi logDetails = do
     writeIORef (wiLogDetails wi) (Just logDetails)
     let (title, subtitle) = logWindowTitle logDetails
-    (wiWindow wi) `set` [ windowTitle := title ]
-    -- TODO: add to gtk2hs
-    objectSetPropertyMaybeString "subtitle" (wiHeaderBar wi) subtitle
+    wiWindow wi `set` [ windowTitle := title ]
+    wiTitle wi `set` [ labelText := title ]
+    wiSubtitle wi `set` [ labelText := subtitle ]
 
 setPage :: MonadIO io
         => WindowInfo
         -> Page
         -> io ()
-setPage wi page = io $ notebookSetCurrentPage (wiNotebook wi) (fromEnum page)
+setPage wi page = io $ stackSetVisibleChildName (wiStack wi) (show page)
 
 displayLog :: WindowInfo
            -> LogDetails
@@ -500,19 +657,19 @@ displayLog :: WindowInfo
            -> Log
            -> RendererResult Participants
            -> B ()
-displayLog wi@(WindowInfo { wiWindow = window
+displayLog wi@WindowInfo { wiWindow = window
                        , wiExport = exportItem
                        , wiViewStatistics = viewStatistics
                        , wiFilterNames = filterNames
                        , wiCanvas = canvas
-                       , wiStatsBook = statsBook
+                       , wiSidebarHeader = sidebarHeader
+                       , wiSidebarStack = sidebarStack
                        , wiStatsPane = statsPane
-                       })
+                       }
            logDetails
            sessionMessages
            systemMessages
-           rr = do
-  io $ do
+           rr = io $ void $ do
     wiSetLogDetails wi logDetails
 
     hiddenRef <- newIORef Set.empty
@@ -534,8 +691,10 @@ displayLog wi@(WindowInfo { wiWindow = window
     viewStatistics `on` checkMenuItemToggled $ do
         active <- checkMenuItemGetActive viewStatistics
         if active
-            then widgetShow statsBook
-            else widgetHide statsBook
+            then do widgetShow sidebarStack
+                    widgetShow sidebarHeader
+            else do widgetHide sidebarStack
+                    widgetHide sidebarHeader
 
     widgetSetSensitivity filterNames True
     onMenuItemActivate filterNames $ do
@@ -546,21 +705,15 @@ displayLog wi@(WindowInfo { wiWindow = window
 
         updateDisplayedLog wi rr'
 
-  return ()
-
-loadPixbuf :: FilePath -> IO (Maybe Pixbuf)
-loadPixbuf filename = do
-  iconName <- getDataFileName $ "data/" ++ filename
-  C.catch (fmap Just (pixbufNewFromFile iconName))
-          (\(GError _ _ msg) -> warn (toString msg) >> return Nothing)
-
-openDialogue :: B ()
-openDialogue = embedIO $ \r -> do
-  chooser <- fileChooserDialogNew Nothing Nothing FileChooserActionOpen
+showOpenDialog :: Window
+               -> B ()
+showOpenDialog window = embedIO $ \r -> do
+  chooser <- fileChooserDialogNew Nothing (Just window) FileChooserActionOpen
              [ ("gtk-cancel", ResponseCancel)
              , ("gtk-open", ResponseAccept)
              ]
-  chooser `set` [ fileChooserLocalOnly := True
+  chooser `set` [ windowModal := True
+                , fileChooserLocalOnly := True
                 ]
 
   chooser `after` response $ \resp -> do
@@ -596,16 +749,50 @@ saveToPDFDialogue wi shapes = do
           RecordedLog _ -> Nothing
           SingleLog p   -> Just $ takeDirectory p
           TwoLogs p _   -> Just $ takeDirectory p
-  maybeM mdirectory $ fileChooserSetCurrentFolder chooser
+  forM_ mdirectory $ fileChooserSetCurrentFolder chooser
 
   chooser `after` response $ \resp -> do
       when (resp == ResponseAccept) $ do
           Just fn <- io $ fileChooserGetFilename chooser
           let (width, height) = diagramDimensions shapes
-          withPDFSurface fn width height $
-            \surface -> renderWith surface $ drawDiagram False shapes
+
+          r <- C.try $ withPDFSurface fn width height $ \surface ->
+              renderWith surface $ drawDiagram False shapes
+          case r of
+              Left (e :: C.IOException) -> do
+                  let title = __ "Couldn't export log as PDF: " ++ show e
+                  displayError wi title Nothing
+              Right () ->
+                  hideError wi
+
       widgetDestroy chooser
 
   widgetShowAll chooser
 
+
+displayError :: WindowInfo
+             -> String
+             -> Maybe String
+             -> IO ()
+displayError wi title mbody = do
+    labelSetMarkup (wiErrorBarTitle wi) . Marquee.toPangoMarkup
+                                        . Marquee.b
+                                        $ Marquee.escape title
+
+    let details = wiErrorBarDetails wi
+    case mbody of
+        Just body -> do
+            labelSetMarkup details . Marquee.toPangoMarkup
+                                   . Marquee.small
+                                   $ Marquee.escape body
+            widgetShow details
+        Nothing   -> widgetHide details
+
+    widgetShow $ wiErrorBar wi
+
+
+hideError :: WindowInfo
+          -> IO ()
+hideError = widgetHide . wiErrorBar
+
 -- vim: sw=2 sts=2
