@@ -46,7 +46,7 @@ showAboutDialog window = do
     dialog `set` [ aboutDialogName := __ "Bustle"
                  , aboutDialogVersion := showVersion version
                  , aboutDialogComments := __ "Someone's favourite D-Bus profiler"
-                 , aboutDialogWebsite := "https://www.freedesktop.org/wiki/Software/Bustle/"
+                 , aboutDialogWebsite := "https://gitlab.freedesktop.org/bustle/bustle#readme"
                  , aboutDialogAuthors := authors
                  , aboutDialogCopyright := "© 2008–2017 Will Thompson, Collabora Ltd. and contributors"
                  , aboutDialogLicense := license
@@ -69,4 +69,5 @@ authors = [ "Will Thompson <will@willthompson.co.uk>"
           , "Alex Merry"
           , "Philip Withnall"
           , "Jonny Lamb"
+          , "Daniel Firth"
           ]
